<?php
/**
 * $Horde: ansel/gallery.php,v 1.57 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Prefs/CategoryManager.php';

/* Redirect to the gallery list if no action has been requested. */
$actionID = Util::getFormData('actionID');
if (is_null($actionID)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

/* Run through the action handlers. */
switch ($actionID) {
case 'add':
    /* Set up the gallery attributes. */
    if ($prefs->getValue('autoname')) {
        $galleryId = md5(microtime());
    } else {
        $galleryId = '';
    }
    $gallery_name = '';
    $gallery_desc = '';
    $gallery_thumbnail_width = $conf['thumbnail']['width'];
    $gallery_thumbnail_height = $conf['thumbnail']['height'];
    $gallery_image_width = $conf['screen']['width'];
    $gallery_image_height = $conf['screen']['height'];
    $gallery_category = '';
    $notification->push('document.gallery.gallery_id.focus();', 'javascript');

    $title = _("Adding A New Gallery");
    break;

case 'addchild':
    // Get the parent and make sure that it exists and that we have
    // permissions to add to it.
    $parentId = Util::getFormData('gallery');
    $parent = &$ansel_shares->getGallery($parentId);
    if (is_a($parent, 'PEAR_Error')) {
        $notification->push($parent->getMessage(), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    } elseif (!$parent->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("You do not have permission to add children to %s."), $parent->get('name')), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    /* Set up the gallery attributes. */
    if ($prefs->getValue('autoname')) {
        $galleryId = md5(microtime());
    } else {
        $galleryId = '';
    }
    $gallery_name = '';
    $gallery_desc = '';
    $gallery_thumbnail_width = $conf['thumbnail']['width'];
    $gallery_thumbnail_height = $conf['thumbnail']['height'];
    $gallery_image_width = $conf['screen']['width'];
    $gallery_image_height = $conf['screen']['height'];
    $gallery_category = '';
    $notification->push('document.gallery.gallery_id.focus();', 'javascript');

    $title = sprintf(_("Adding A Subgallery to %s"), $parent->get('name'));
    break;

case 'downloadzip':
    $galleryId = Util::getFormData('gallery');
    $gallery = &$ansel_shares->getGallery($galleryId);
    if (!Auth::getAuth() || is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
        $name = is_a($gallery, 'PEAR_Error') ? $galleryId : $gallery->get('name');
        $notification->push(sprintf(_("Access denied downloading %s."), $name), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    // Don't generate an empty zipfile.
    if ($gallery->get('num-images') == 0) {
        $notification->push(sprintf(_("There are no images in %s to download."), $name), 'horde.message');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    /* Try to close off the current session to avoid locking it while
     * the gallery is downloading. */
    @session_write_close();

    require_once 'Horde/Compress.php';

    $images = $gallery->listImages();

    $zipfiles = array();
    foreach ($images as $id => $name) {
        $image = &$ansel_shares->getImage($id);
        $zipfiles[] = array('data' => $image->raw(), 'name' => $image->get('filename'));
    }

    $zip = &Horde_Compress::singleton('zip');
    $body = $zip->compress($zipfiles);
    $browser->downloadHeaders($galleryId . '.zip', 'application/zip', false, strlen($body));
    echo $body;
    break;

case 'modify':
    $galleryId = Util::getFormData('gallery');
    $gallery = &$ansel_shares->getGallery($galleryId);

    if ($gallery) {
        /* Set up the gallery attributes. */
        $gallery_name = $gallery->get('name');
        $gallery_desc = $gallery->get('desc');
        $gallery_thumbnail_width = $gallery->get('thumbnail_width');
        $gallery_thumbnail_height = $gallery->get('thumbnail_height');
        $gallery_image_width = $gallery->get('image_width');
        $gallery_image_height = $gallery->get('image_height');
        $gallery_category = $gallery->get('category');

        $title = sprintf(_("Modifying: %s"), $gallery_name);
    } else {
        $title = _("Unknown gallery");
    }
    break;

case 'save':
    /* Check general permissions. */
    if (!Auth::isAdmin() &&
        ($GLOBALS['perms']->exists('ansel') &&
         !$GLOBALS['perms']->hasPermission('ansel', Auth::getAuth(), PERMS_EDIT))) {
        $notification->push(_("Access denied editing galleries."), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    /* Get the form values. */
    $galleryId = Util::getFormData('gallery');
    $gallery_name = Util::getFormData('gallery_name');
    $gallery_desc = Util::getFormData('gallery_desc');
    $gallery_thumbnail_width = Util::getFormData('gallery_thumbnail_width');
    $gallery_thumbnail_height = Util::getFormData('gallery_thumbnail_height');
    $gallery_image_width = Util::getFormData('gallery_image_width');
    $gallery_image_height = Util::getFormData('gallery_image_height');
    $gallery_category = Util::getFormData('gallery_category');

    /* Make sure thumbnail width is within set limits */
    if ($gallery_thumbnail_width < $conf['thumbnail']['width_min']) {
        $gallery_thumbnail_width = $conf['thumbnail']['width_min'];
        $notification->push(sprintf(_("Specified thumbnail width is too small, value has been set to %s"), $conf['thumbnail']['width_min']), 'horde.warning');
    } elseif ($gallery_thumbnail_width > $conf['thumbnail']['width_max']) {
        $gallery_thumbnail_width = $conf['thumbnail']['width_max'];
        $notification->push(sprintf(_("Specified thumbnail width is too large, value has been set to %s"), $conf['thumbnail']['width_max']), 'horde.warning');
    }

    /* Make sure thumbnail height is within set limits */
    if ($gallery_thumbnail_height < $conf['thumbnail']['height_min']) {
        $gallery_thumbnail_height = $conf['thumbnail']['height_min'];
        $notification->push(sprintf(_("Specified thumbnail height is too small, value has been set to %s"), $conf['thumbnail']['height_min']), 'horde.warning');
    } elseif ($gallery_thumbnail_height > $conf['thumbnail']['height_max']) {
        $gallery_thumbnail_height = $conf['thumbnail']['height_max'];
        $notification->push(sprintf(_("Specified thumbnail height is too large, value has been set to %s"), $conf['thumbnail']['height_max']), 'horde.warning');
    }

    /* Make sure screen image width is within set limits */
    if ($gallery_image_width < $conf['screen']['width_min']) {
        $gallery_image_width = $conf['screen']['width_min'];
        $notification->push(sprintf(_("Specified screen image width is too small, value has been set to %s"), $conf['screen']['width_min']), 'horde.warning');
    } elseif ($gallery_image_width > $conf['screen']['width_max']) {
        $gallery_image_width = $conf['screen']['width_max'];
        $notification->push(sprintf(_("Specified screen image width is too large, value has been set to %s"), $conf['screen']['width_max']), 'horde.warning');
    }

    /* Make sure screen image height is within set limits */
    if ($gallery_image_height < $conf['screen']['height_min']) {
        $gallery_image_height = $conf['screen']['height_min'];
        $notification->push(sprintf(_("Specified screen image height is too small, value has been set to %s"), $conf['screen']['height_min']), 'horde.warning');
    } elseif ($gallery_image_height > $conf['screen']['height_max']) {
        $gallery_image_height = $conf['screen']['height_max'];
        $notification->push(sprintf(_("Specified screen image height is too large, value has been set to %s"), $conf['screen']['height_max']), 'horde.warning');
    }

    if ($ansel_shares->exists($ansel_shares->getName($galleryId))) {
        /* Modifying an existing gallery. */
        $gallery = &$ansel_shares->getGallery($galleryId);
        if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
            $name = is_a($gallery, 'PEAR_Error') ? $galleryId : $gallery->get('name');
            $notification->push(sprintf(_("Access denied saving gallery %s."), $name), 'horde.error');
        } else {
            // Don't allow the display name to be nulled out.
            if ($gallery_name) {
                $gallery->set('name', $gallery_name);
            }

            // If thumbnail sizes have changed, delete vfs thumbnail
            // cache.
            if ($gallery_thumbnail_width != $gallery->get('thumbnail_width') ||
                $gallery_thumbnail_height != $gallery->get('thumbnail_height')) {
                $images = $gallery->listImages();
                foreach ($images as $id => $name) {
                    $image = &$ansel_shares->getImage($id);
                    $image->deleteCache('thumb');
                }
            }

            // If screen image sizes have changed, delete vfs
            // screen image cache.
            if ($gallery_image_width != $gallery->get('image_width') ||
                $gallery_image_height != $gallery->get('image_height')) {
                $images = $gallery->listImages();
                foreach ($images as $id => $name) {
                    $image = &$ansel_shares->getImage($id);
                    $image->deleteCache('screen');
                }
            }

            $gallery->set('desc', $gallery_desc);
            $gallery->set('thumbnail_width', $gallery_thumbnail_width);
            $gallery->set('thumbnail_height', $gallery_thumbnail_height);
            $gallery->set('image_width', $gallery_image_width);
            $gallery->set('image_height', $gallery_image_height);
            $gallery->set('category', $gallery_category);
            $gallery->save();
            $notification->push(_("The gallery was saved."), 'horde.success');
        }
    } else {
        /* Creating a new gallery. */
        $galleryId = Util::getFormData('gallery_id');

        /* Is this a subgallery? */
        $parentId = Util::getFormData('parent');
        if ($parentId && $galleryId) {
            $parent = &$ansel_shares->getGallery($parentId);
            if (is_a($parent, 'PEAR_Error')) {
                $notification->push($parent->getMessage(), 'horde.error');
                header('Location: ' . Horde::applicationUrl('list.php', true));
                exit;
            } elseif (!$parent->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
                $notification->push(sprintf(_("You do not have permission to add children to %s."), $parent->get('name')), 'horde.error');
                header('Location: ' . Horde::applicationUrl('list.php', true));
                exit;
            }

            /* Passed permission checks, so update the id so it's
             * listed as a child. */
            $galleryId = $ansel_shares->getName($parentId) . ':' . $galleryId;
        }

        /* Make sure the new id is unique. */
        if (!$galleryId || $ansel_shares->exists($galleryId)) {
            $notification->push(sprintf(_("'%s' is an invalid unique name; it may already be taken. Please choose a different unique name."), $galleryId), 'horde.warning');
            $actionId = 'add';
            $title = _("Adding A New Gallery");
            break;
        }

        /* Require a display name. */
        if (!$gallery_name) {
            $notification->push(_("You must provide a display name for your new gallery."), 'horde.warning');
            $actionId = 'add';
            $title = _("Adding A New Gallery");
            break;
        }

        /* Creating a new gallery. */
        $perm = ($parentId) ? $parent->getPermission() : '';
        $gallery = &Ansel::createGallery($galleryId, array('name' => $gallery_name,
                                                           'desc' => $gallery_desc,
                                                           'thumbnail_width' => $gallery_thumbnail_width,
                                                           'thumbnail_height' => $gallery_thumbnail_height,
                                                           'image_width' => $gallery_image_width,
                                                           'image_height' => $gallery_image_height,
                                                           'category' => $gallery_category
                                                           ), $perm);
        if (is_a($gallery, 'PEAR_Error')) {
            $galleryId = null;
            $error = sprintf(_("The gallery \"%s\" couldn't be created: %s"), $gallery_name, $gallery->getMessage());
            Horde::logMessage($error, __FILE__, __LINE__, PEAR_LOG_ERR);
            $notification->push($error, 'horde.error');
        } else {
            $galleryId = $gallery->getId();
            $msg = sprintf(_("The gallery \"%s\" was created successfully."), $gallery_name);
            Horde::logMessage($msg, __FILE__, __LINE__, PEAR_LOG_DEBUG);
            $notification->push($msg, 'horde.success');
        }
    }

    /* Return to viewing the gallery or the gallery list, whichever is
     * appropriate. */
    if (empty($galleryId)) {
        $url = 'list.php';
    } else {
        $url = Util::addParameter('view.php', 'gallery', $galleryId);
    }
    header('Location: ' . Horde::applicationUrl($url, true));
    exit;
    break;

case 'delete':
    /* Delete the gallery if we're provided with a valid galleryId. */
    $galleryId = Util::getFormData('gallery');
    if (isset($galleryId)) {
        $gallery = $ansel_shares->getGallery($galleryId);
        if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
            $notification->push(sprintf(_("Access denied deleting gallery %s."), $gallery->get('name')), 'horde.error');
        } elseif ($ansel_shares->hasChildren($gallery)) {
            $notification->push(sprintf(_("You cannot delete %s, it contains nested galleries."), $gallery->get('name')), 'horde.warning');
        } else {
            $imageList = $gallery->listImages();
            $result = $ansel_shares->removeShare($gallery);
            if (!is_a($result, 'PEAR_Error')) {
                $notification->push(sprintf(_("Successfully deleted %s."), $gallery->get('name')), 'horde.success');
                if (count($imageList)) {
                    foreach ($imageList as $id => $image) {
                        $gallery->removeImage($id, true);
                    }
                }
            } else {
                $notification->push(sprintf(_("There was a problem deleting %s: %s."), $gallery->get('name'), $result->getMessage()), 'horde.error');
            }
        }
    }

    /* Return to the gallery list. */
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
    break;

case 'empty':
    $galleryId = Util::getFormData('gallery');
    $gallery = &$ansel_shares->getGallery($galleryId);

    /* Make sure the requested gallery exists. */
    if (is_a($gallery, 'PEAR_Error')) {
        $notification->push($gallery->getMessage(), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    // Make sure we have permission to empty the gallery.
    if (!$gallery->hasPermission(Auth::getAuth(), PERMS_EDIT)) {
        $notification->push(sprintf(_("You do not have permission to empty %s."), $gallery->get('name')), 'horde.error');
        header('Location: ' . Horde::applicationUrl('list.php', true));
        exit;
    }

    $images = $gallery->listImages();
    foreach ($images as $image) {
        $gallery->removeImage($image, true);
    }
    $gallery->set('num-images', 0, true);

    $url = Util::addParameter('view.php', 'gallery', $galleryId);
    header('Location: ' . Horde::applicationUrl($url, true));
    exit;
    break;

default:
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

require ANSEL_TEMPLATES . '/common-header.inc';
require ANSEL_TEMPLATES . '/menu.inc';
require ANSEL_TEMPLATES . '/gallery/gallery.inc';
require $registry->get('templates', 'horde') . '/common-footer.inc';
