<?php
/**
 * Ansel external API interface.
 *
 * $Horde: ansel/lib/api.php,v 1.6 2004/12/07 16:09:11 chuck Exp $
 *
 * This file defines Ansel's external API interface. Other
 * applications can interact with Ansel through this API.
 *
 * @package Ansel
 */

$_services['commentCallback'] = array(
    'args' => array('image_id' => 'string'),
    'type' => 'boolean'
);

$_services['saveImage'] = array(
    'args' => array('app'          => 'string',
                    'gallery_id'   => 'string',
                    'image'        => 'array',
                    'default'      => 'boolean',
                    'gallery_data' => 'array',
                    'perm'         => 'object'),
    'type' => 'array'
);

$_services['removeImage'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string',
                    'image_id'   => 'string'),
    'type' => 'int'
);

$_services['removeGallery'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string'),
    'type' => 'int'
);

$_services['getImageUrl'] = array(
    'args' => array('app'        => 'string',
                    'image_id'   => 'string',
                    'view'       => 'string',
                    'full'       => 'boolean'),
    'type' => 'string'
);

$_services['numImages'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string',
                    'image_id'   => 'string'),
    'type' => 'int'
);

$_services['getDefaultImage'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string'),
    'type' => 'string'
);

$_services['listGalleries'] = array(
    'args' => array('app'        => 'string',
                    'perm'       => 'integer',
                    'parent'     => 'string',
                    'allLevels'  => 'string',
                    'from'       => 'integer',
                    'count'      => 'integer'),
    'type' => 'string'
);

$_services['listImages'] = array(
    'args' => array('app'        => 'string',
                    'gallery_id' => 'string',
                    'perm'       => 'integer',
                    'view'       => 'string',
                    'full'       => 'boolean',
                    'from'       => 'integer',
                    'count'      => 'integer'),
    'type' => 'string'
);


/**
 * Callback for Agora comments.
 */
function _ansel_commentCallback($image_id)
{
    if (!$GLOBALS['conf']['comments']['allow']) {
        return false;
    }

    require_once dirname(__FILE__) . '/base.php';

    $image = &$GLOBALS['ansel_shares']->getImage($image_id);
    if (!$image || is_a($image, 'PEAR_Error')) {
        return false;
    }

    return true;
}

/**
 * Stores an image in a gallery adn returns gallery and image data.
 *
 * @param integer $app           Application used if null then use default.
 * @param integer $gallery_id    Unique id to be used as gallery uniqueID.
 * @param array   $image         Image data array (ex. from
 *                               Horde_Form_Type_image).
 * @param integer $default       Set this image as default in the gallery?
 * @param array   $gallery_data  Gallery parameters (like image dimensions).
 * @param Perms   $perm          The permissions for the gallery if the
 *                               defaults are not desirable.
 *
 * @return mixed  False if error, or id of an array of image/gallery data.
 */
function _ansel_saveImage($app = null, $gallery_id, $image, $default = false,
                          $gallery_data = array(), $perm = null)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized
     * in base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = &new Ansel_Gallery($app);
    }

    if (getimagesize($image['file']) === false) {
        return PEAR::raiseError(_("The file you uploaded does not appear to be a valid image."));
    } elseif ($GLOBALS['ansel_shares']->exists($gallery_id)) {
        $gallery = $GLOBALS['ansel_shares']->getGallery($gallery_id);
        if (is_a($gallery, 'PEAR_Error')) {
            return $gallery;
        }
        foreach ($gallery_data as $key => $value) {
            $gallery->set($key, $value);
        }
        $gallery->save();
    } else {
        if (empty($gallery_data['name'])) {
            $gallery_data['name'] = $gallery_id;
        }
        $gallery = &Ansel::createGallery($gallery_id, $gallery_data, $perm);
        if (is_a($gallery, 'PEAR_Error')) {
            return $gallery;
        }
    }

    $image_data = array(
        'filename' => $image['name'],
        'description' => $image['name'],
        'data' => file_get_contents($image['file']),
        'type' => $image['name']['type'],
    );

    $image_id = $gallery->addImage($image_data);
    if (is_a($image_id, 'PEAR_Error')) {
        return $image_id;
    }

    $image = &$GLOBALS['ansel_shares']->getImage($image_id);
    if (is_a($image, 'PEAR_Error')) {
        return $image;
    }

    $image->createView('thumb');
    $image->createView('screen');

    if ($default) {
        $gallery->set('default', $image_id, true);
        $gallery->set('default-type', 'manual', true);
        $gallery->save();
    }

    $data = array(
        'image-id' => (int)$image_id,
        'gallery-id' => (int)$gallery->getId(),
        'num-images' => (int)$gallery->data['num-images'],
    );

    return $data;
}

/**
 * Removes an image from a gallery.
 *
 * @param string $app         Application used if null then use default.
 * @param string $gallery_id  The id of gallery.
 * @param string $image_id    The id of image to remove.
 */
function _ansel_removeImage($app = null, $gallery_id, $image_id)
{
    global $ansel_shares;

    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $ansel_shares = &new Ansel_Gallery($app);
    }

    $gallery = &$ansel_shares->getGallery($gallery_id);

    return $gallery->removeImage($image_id);
}

/**
 * Removes a gallery and its images.
 *
 * @param string $app         Application used if null then use default.
 * @param string $gallery_id  The id of gallery.
 */
function _ansel_removeGallery($app = null, $gallery_id)
{
    global $ansel_shares;

    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $ansel_shares = &new Ansel_Gallery($app);
    }

    $gallery = $ansel_shares->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error') ||
        !$gallery->hasPermission(Auth::getAuth(), PERMS_DELETE)) {
        return PEAR::raiseError(sprintf(_("Access denied deleting gallery %s."),
                                        $gallery->get('name')));
    } else {
        $imageList = $gallery->listImages();
        $result = $ansel_shares->removeShare($gallery);
        if (!is_a($result, 'PEAR_Error')) {
            if (count($imageList)) {
                foreach ($imageList as $id => $image) {
                    $gallery->removeImage($id, true);
                }
            }
            return true;
        } else {
            return PEAR::raiseError(sprintf(_("There was a problem deleting %s: %s"),
                                            $gallery->get('name'),
                                            $result->getMessage()));
        }
    }
}

/**
 * Returns the number of images in a gallery.
 *
 * @param integer $app         Application used; if null then use default.
 * @param string  $gallery_id  The id of gallery or image.
 *
 * @return integer  The number of images in the gallery.
 */
function _ansel_numImages($app = null, $gallery_id = null)
{
    global $ansel_shares;

    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $ansel_shares = &new Ansel_Gallery($app);
    }

    $gallery = &$ansel_shares->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error')) {
        return 0;
    } else {
        return (int)$gallery->data['num-images'];
    }
}

/**
 * Returns the default image id of a gallery.
 *
 * @param integer $app         Application used.
 * @param string  $gallery_id  The id of gallery or image.
 *
 * @return integer  The default image id.
 */
function _ansel_getDefaultImage($app = null, $gallery_id)
{
    global $ansel_shares;

    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $ansel_shares = &new Ansel_Gallery($app);
    }

    $gallery = $ansel_shares->getGallery($gallery_id);
    if (is_a($gallery, 'PEAR_Error')) {
        return $gallery;
    } else {
        return $gallery->getDefaultImage();
    }
}

/**
 * Returns image URL.
 *
 * @param integer $app       Application used.
 * @param string  $image_id  The id of the image.
 * @param string  $view      The view ('screen', 'thumb', or 'full') to show.
 *                           Defaults to screen.
 * @param boolean $full      Return a path that includes the server name?
 *
 * @return string  The image path.
 */
function _ansel_getImageUrl($app = null, $image_id, $view = 'screen', $full = false)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = &new Ansel_Gallery($app);
    }

    return Ansel::getImageUrl($image_id, $view, $full);
}

/**
 * Returns a list of all galleries.
 *
 * @param string  $app        Horde application used.
 * @param integer $perm       The level of permissions to require for a gallery
 *                            to be returned.
 * @param string  $parent     The parent share to start searching from.
 * @param boolean $allLevels  Return all levels, or just the direct children of
 *                            $parent? Defaults to all levels.
 * @param integer $from       The gallery to start listing at.
 * @param integer $count      The number of galleries to return.
 *
 * @return array  A 2-dimensional array with gallery ids and names.
 */
function _ansel_listGalleries($app = null, $perm = PERMS_SHOW,
                              $owneronly = false, $parent = DATATREE_ROOT,
                              $allLevels = true, $from = 0, $count = 0)
{
    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $GLOBALS['ansel_shares'] = &new Ansel_Gallery($app);
    }

    $galleries = Ansel::listGalleries($perm, $owneronly, $parent, $allLevels, $from, $count);

    $gallerylist = array();

    foreach ($galleries as $groupId => $group) {
        foreach ($group['galleries'] as $id => $gallery) {
            $gallerylist[$gallery->getId()]['name'] = $gallery->get('name');
        }
    }

    return $gallerylist;
}

/**
 * Returns a list of all images in a gallery.
 *
 * The return has the URL because in a lot of cases you'll want the url
 * also. Using api call getImageURL results in a lot of overhead when
 * e.g. generating a select list.
 *
 * @param string  $app         Horde application used.
 * @param string  $gallery_id  Gallery to get images from.
 * @param integer $perm        The level of permissions to require for a
 *                             gallery to return it.
 * @param string  $view        Viewsize to generate URLs for.
 * @param boolean $full        Return URL of full image.
 * @param integer $from        Start image
 * @param integer $count       End image.
 *
 * @return array  Two dimensional array with image names ids (key) and urls.
 */
function _ansel_listImages($app = null, $gallery_id = '', $perm = PERMS_SHOW,
                           $view = 'screen', $full = false, $from = 0, $count = 0)
{
    global $ansel_shares;

    require_once dirname(__FILE__) . '/base.php';

    /* If no app is given use Ansel's own gallery which is initialized in
       base.php */
    if (!is_null($app)) {
        $ansel_shares = &new Ansel_Gallery($app);
    }

    /* Determine the default gallery when none is given. The first gallery in
       the list is the default gallery. */
    if ($gallery_id == '') {
        $galleries = Ansel::listGalleries($perm);
        $keys = array_keys($galleries);
        $gallery_names = array_keys($galleries[$keys[0]]['galleries']);
        $gallery_id = $gallery_names[0];
    }

    $gallery = $ansel_shares->getGallery($gallery_id);
    $images = $gallery->listImages();

    $counter = 0;
    $imagelist = array();
    foreach ($images as $id => $name) {
        $image = $ansel_shares->getImage($id);
        $imagelist[$id]['name'] = $image->get('filename');
        $imagelist[$id]['url'] = Ansel::getImageUrl($id, $view, $full);
    }

    return $imagelist;
}
