#!/usr/local/bin/php
<?php
/**
 * $Horde: ansel/scripts/upgrades/2004-01-15_update_gallery_format.php,v 1.2 2004/12/07 16:09:12 chuck Exp $
 *
 * This is a script to migrate old galleries to the new Ansel storage
 * format. You must update Ansel before running this script.
 *
 * If you have galleries that predate migrate_galleries.php, you must
 * run migrate_galleries.php before running this script.
 *
 * !!! IMPORTANT!!! Make sure you run this script with a user that has
 * write-access to your VFS.  It would probably be a good idea to
 * back-up your database first, too...just in case.
 */

/**
 ** Set this to true if you want DB inserts and VFS renames done.
 **/
$live = false;

@define('AUTH_HANDLER', true);
@define('ANSEL_BASE', dirname(__FILE__) . '/..');
@define('HORDE_BASE', ANSEL_BASE . '/..');

// Do CLI checks and environment setup first.
require_once HORDE_BASE . '/lib/core.php';
require_once 'Horde/CLI.php';

// Make sure no one runs this from the web.
if (!Horde_CLI::runningFromCLI()) {
    exit("Must be run from the command line\n");
}

// Load the CLI environment - make sure there's no time limit, init
// some variables, etc.
Horde_CLI::init();

require_once ANSEL_BASE . '/lib/base.php';

$vfs = &VFS::singleton($conf['vfs']['type'], Horde::getDriverConfig('vfs', $conf['vfs']['type']));

$driver = $conf['datatree']['driver'];
$datatree = &DataTree::singleton($driver, array_merge(Horde::getDriverConfig('datatree', $driver),
                                                      array('group' => 'horde.shares.ansel')));

$galleries = $datatree->get(DATATREE_FORMAT_FLAT, DATATREE_ROOT, true);
foreach ($galleries as $galleryId => $galleryName) {
    $gallery = &$ansel_shares->getGallery($galleryName);
    $attributes = $datatree->getAttributes($galleryId);
    $images = array();
    foreach ($attributes as $attr) {
        if (substr($attr['name'], 0, 6) == 'images') {
            $images[substr($attr['name'], 7)][$attr['key']] = $attr['value'];
            unset($gallery->data[$attr['name']]);
        }
    }

    foreach ($gallery->data as $key => $val) echo "$key: $val\n";

    $vfspath = sprintf('.horde/ansel/%s', $gallery->getName());
    echo "vfspath: $vfspath\n";

    foreach ($images as $image) {
        echo '.';
        if ($live) {
            $imageName = $gallery->addImage($image);
            $newImage = &$ansel_shares->getImage($imageName);
            $vfs->rename($vfspath, $image['id'] . '_full', $vfspath, $newImage->getId() . '_full');
            $vfs->deleteFile($vfspath, $image['id'] . '_thumb');
            $vfs->deleteFile($vfspath, $image['id'] . '_screen');
            if ($gallery->get('default') == $image['id']) {
                $gallery->set('default', $newImage->getId(), true);
            }
        }
    }

    if ($live) {
        $gallery->set('num-images', sizeof($images), true);
        echo "\nimages: " . $gallery->get('num-images') . "\n\n";
    }
}
