/**
 * Utility functions to use when cropping an image.
 *
 * Adapted from javascript in W-Nailer (http://wnailer.vddh.com/).
 */

/**
 * Page variables.
 */
var mousePosX = 0;
var mousePosY = 0;
var cropTop = 150;
var origX = 0;
var origY = 0;
var cropper = '';
var zoom = '';
var orig = '';
var extra = <?php echo $GLOBALS['browser']->isBrowser('msie') ? 1 : 0 ?>;
var origCropperX = 0;
var origCropperY = 0;
var origWidth = 0;
var origHeight = 0;
var minWidth = 30;
var minHeight = 30;
var resize = false;

/**
 * Set up page variables once everything has loaded.
 */
function init()
{
    cropper = document.getElementById('cropper');
    zoom = document.getElementById('zoom');
    orig = document.getElementById('original');
    document.onmousemove = getMouseXY;
}

/**
 * Update the original image and the cropper selection on a zoom
 * operation.
 */
function imageSize()
{
    // Get the original width & height of the image.
    width = document.getElementById('or_width').value;
    height = document.getElementById('or_height').value;

    // Get previous zoom percentage.
    zoom.previous = width / orig.width;

    if (zoom.previous < zoom.value) {
        // Zoom in.

        // Resize image, based on zoom & original size.
        orig.width = Math.ceil(width / zoom.value);
        orig.height = Math.ceil(height / zoom.value);

        // Resize cropper.
        cropper.style.width = Math.ceil(cropper.style.width.substring(0, cropper.style.width.length - 2) * zoom.previous) / zoom.value + 'px';
        cropper.style.height = Math.ceil(cropper.style.height.substring(0, cropper.style.height.length - 2) * zoom.previous) / zoom.value + 'px';

        // Re-position cropper.
        if (!cropper.style.left) {
            cropper.style.left = 10 + extra + 'px';
            cropper.style.top = cropTop + extra + 'px';
        } else {
            cropper.style.left = (cropper.style.left.substring(0, cropper.style.left.length - 2) - 10 - extra) * zoom.previous / zoom.value + 10 + extra + 'px'
            cropper.style.top = (cropper.style.top.substring(0, cropper.style.top.length - 2) - cropTop - extra) * zoom.previous / zoom.value + cropTop + extra + 'px'
        }
    } else {
        // Zoom out.

        // Resize image.
        orig.width = Math.ceil(width / zoom.value);
        orig.height = Math.ceil(height / zoom.value);

        // Resize cropper.
        cropper.style.width = Math.ceil(cropper.style.width.substring(0, cropper.style.width.length - 2) * zoom.previous) / zoom.value + 'px';
        cropper.style.height = Math.ceil(cropper.style.height.substring(0, cropper.style.height.length - 2) * zoom.previous) / zoom.value + 'px';

        // Re-position cropper.
        cropper.style.left = (cropper.style.left.substring(0, cropper.style.left.length - 2) - 10 - extra) * zoom.previous / zoom.value + 10 + extra + 'px';
        cropper.style.top = (cropper.style.top.substring(0, cropper.style.top.length - 2) - cropTop - extra) * zoom.previous / zoom.value + cropTop + extra + 'px';
    }

    document.getElementById('cropW').value = Math.floor(cropper.style.width.substring(0, cropper.style.width.length - 2) * zoom.value);
    document.getElementById('cropH').value = Math.floor(cropper.style.height.substring(0, cropper.style.height.length - 2) * zoom.value);
}


/**
 * Set the mouseX and mouseY coords.
 */
function getMouseXY(e)
{
    if (document.all) {
        mousePosX = event.clientX; + document.body.scrollLeft;
        mousePosY = event.clientY; + document.body.scrollTop;
    } else {
        mousePosX = e.pageX;
        mousePosY = e.pageY;
    }

    if (!cropper.style.top) {
        cropper.style.left = 10 + extra + 'px';
        cropper.style.top = cropTop + extra + 'px';
    }

    // Define the cropper area to set the mouse cursor by.
    x1 = cropper.style.left.substring(0, cropper.style.left.length - 2) * 1;
    x2 = x1 + cropper.style.width.substring(0, cropper.style.width.length - 2) * 1;
    y1 = cropper.style.top.substring(0, cropper.style.top.length - 2) * 1;
    y2 = y1 + cropper.style.height.substring(0, cropper.style.height.length - 2) * 1;

    // Is the cursor above the cropper?
    if (mousePosX > x1 && mousePosX < x2 && mousePosY > y1 && mousePosY < y2) {
        document.body.style.cursor = 'move';
    } else {
        document.body.style.cursor = 'default';
    }
}

/**
 * Start a drag of the cropper layer.
 */
function startCropperDrag()
{
    if (!cropper) {
        return false;
    }

    origX = mousePosX;
    origY = mousePosY;

    origCropperX = Math.abs(cropper.style.left.substring(0, cropper.style.left.length - 2));
    origCropperY = Math.abs(cropper.style.top.substring(0, cropper.style.top.length - 2));

    document.onmousemove = (resize ? onResize : onCropperDrag);
}

/**
 * Clean up after moving the cropper selection.
 */
function endCropperDrag()
{
    origX = 0;
    origY = 0;

    document.onmousemove = getMouseXY;
}

/**
 * Called when the mouse moves while moving the cropper - update the
 * layer and the coordinate info.
 */
function onCropperDrag(e)
{
    getMouseXY(e);
    diffX = mousePosX - origX;
    diffY = mousePosY - origY;

    cropper.style.left = origCropperX + diffX + 'px';
    cropper.style.top = origCropperY + diffY + 'px';

    // Lock movement to image borders; left/top boundary.
    // Left.
    if ((origCropperX + diffX) < 10) {
        cropper.style.left = (10 + extra) + 'px';
    }
    // Top.
    if ((origCropperY + diffY) < cropTop) {
        cropper.style.top = (cropTop + extra) + 'px';
    }

    // Width of cropper.
    origWidth = Math.abs(cropper.style.width.substring(0, cropper.style.width.length - 2));

    // Height of cropper.
    origHeight = Math.abs(cropper.style.height.substring(0, cropper.style.height.length - 2));

    // Lock movement to image borders; right/bottom boundary.
    // Right.
    if ((origCropperX + diffX + origWidth) > (10 + orig.width)) {
        cropper.style.left = (10 + orig.width - origWidth + extra) + 'px';
    }
    // Bottom.
    if ((origCropperY + diffY + origHeight) > (cropTop + orig.height)) {
        cropper.style.top = (cropTop + orig.height - origHeight + extra)  + 'px';
    }

    // Output coordinates to the form.
    document.getElementById('x1').value = Math.floor((cropper.style.left.substring(0, cropper.style.left.length - 2) - 10 - extra) * zoom.value);
    document.getElementById('y1').value = Math.floor((cropper.style.top.substring(0, cropper.style.top.length - 2) - cropTop - extra) * zoom.value);

    document.getElementById('cropW').value = Math.floor(cropper.style.width.substring(0, cropper.style.width.length - 2) * zoom.value);
    document.getElementById('cropH').value = Math.floor(cropper.style.height.substring(0, cropper.style.height.length - 2) * zoom.value);

    document.getElementById('x2').value = 1 * document.getElementById('x1').value + 1 * document.getElementById('cropW').value;
    document.getElementById('y2').value = 1 * document.getElementById('y1').value + 1 * document.getElementById('cropH').value;
}

/**
 * Initiate a resize of the cropper layer.
 */
function startResize()
{
    if (!cropper) {
        return false;
    }

    origX = mousePosX;
    origY = mousePosY;

    origWidth = Math.abs(cropper.style.width.substring(0, cropper.style.width.length - 2));
    origHeight = Math.abs(cropper.style.height.substring(0, cropper.style.height.length - 2));

    resize = true;
}

/**
 * End a cropper resize, clean up.
 */
function endResize()
{
    origX = 0;
    origY = 0;

    resize = false;
    document.onmousemove = getMouseXY;
}

/**
 * Called when the mouse moves during a resize. Updates the layer and
 * coordinate information.
 */
function onResize(e)
{
    getMouseXY(e);
    diffX = mousePosX - origX;
    diffY = mousePosY - origY;

    if (origWidth + diffX <= minWidth) cropper.style.width  = minWidth + 'px'; // minimum width exceeded
    else if ( (origCropperX + diffX + origWidth) > (10 + orig.width) )cropper.style.width  = (10 + orig.width) - origCropperX + extra + 'px'; // resize to border
    else cropper.style.width  = origWidth + diffX + 'px'; // normal resize

    if (origHeight + diffY <= minHeight) {
        // Minimum height underflowed.
        cropper.style.height = minHeight + 'px';
    } else if ((origCropperY + diffY + origHeight) > (cropTop + orig.height)) {
        // Resize to border.
        cropper.style.height = (cropTop + orig.height) - origCropperY + extra + 'px';
    } else {
        // Normal resize.
        cropper.style.height = origHeight + diffY + 'px';
    }

    // Output position to browser.
    document.getElementById('cropW').value = Math.floor(cropper.style.width.substring(0, cropper.style.width.length - 2) * zoom.value)
    document.getElementById('cropH').value = Math.floor(cropper.style.height.substring(0, cropper.style.height.length - 2) * zoom.value)

    document.getElementById('x2').value = Math.floor(zoom.value * document.getElementById('x1').value + 1*document.getElementById('cropW').value)
    document.getElementById('y2').value = Math.floor(zoom.value * document.getElementById('y1').value + 1*document.getElementById('cropH').value)
}
