<?php
/**
 * $Horde: ansel/view.php,v 1.71 2005/01/03 14:34:47 jan Exp $
 *
 * Copyright 2001-2005 Chuck Hagenbuch <chuck@horde.org>
 *
 * See the enclosed file COPYING for license information (GPL).  If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 */

@define('ANSEL_BASE', dirname(__FILE__));
require_once ANSEL_BASE . '/lib/base.php';
require_once 'Horde/Text.php';
require_once 'Horde/Text/Filter.php';

/* If we aren't provided with a gallery, redirect to the gallery list. */
$galleryId = Util::getFormData('gallery');
$actionID = Util::getFormData('actionID');

if (!isset($galleryId)) {
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$gallery = &$ansel_shares->getGallery($galleryId);
if (is_a($gallery, 'PEAR_Error') || !$gallery->hasPermission(Auth::getAuth(), PERMS_READ)) {
    $notification->push(_("You do not have permission to view this gallery."), 'horde.error');
    header('Location: ' . Horde::applicationUrl('list.php', true));
    exit;
}

$print_view = (bool)Util::getFormData('print');

/* If we aren't supplied with a page number, default to page 0. */
$page = Util::getFormData('page', 0);
$gpage = Util::getFormData('gpage', 0);

switch ($actionID) {
case 'viewimage':
    $image_id = Util::getFormData('image');
    $title = _("Image Details");

    /* If the requested gallery doesn't exist, display an error
     * message. */
    if (is_a($gallery, 'PEAR_Error')) {
        require ANSEL_TEMPLATES . '/common-header.inc';
        require ANSEL_TEMPLATES . '/menu.inc';
        require ANSEL_TEMPLATES . '/view/no-gallery.inc';
    } else {
        /* Retrieve image details. */
        $image = &$ansel_shares->getImage($image_id);

        /* If the requested image doesn't exist, display an error
         * message. */
        if (!$image) {
            $notification->push(_("Image not found."), 'horde.error');
            header('Location: ' . Horde::applicationUrl('list.php', true));
            exit;
        }

        $title = $gallery->get('name') . ' :: ' . $image->get('filename');
        if (!$print_view) {
            Horde::addScriptFile('popup.js', 'horde', true);
        }
        require ANSEL_TEMPLATES . '/common-header.inc';

        if ($print_view) {
            require_once $registry->get('templates', 'horde') . '/javascript/print.js';
        } else {
            $print_link = Horde::url(Util::addParameter('view.php', array('gallery' => $galleryId, 'image' => $image_id, 'actionID' => 'viewimage', 'print' => 'true')));
            require ANSEL_TEMPLATES . '/menu.inc';

            /* See if we have any parents to go up to. */
            $parent_list = $gallery->getParents();
            $parents = array();
            foreach($parent_list as $id) {
                $p = $ansel_shares->getGallery($id);
                $parents[] = Horde::link(Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $p->getId())), $p->get('name'), 'smallheader') . $p->get('name') . '</a>';
            }
            if (count($parents)) {
                require ANSEL_TEMPLATES . '/view/parents.inc';
            }

            /* Find the next and previous images, if there are
             * any. Make sure to account for gaps in the ids. */
            $imageList = array_keys($gallery->listImages());
            $revList = array_flip($imageList);
            if (isset($imageList[$revList[$image_id] + 1])) {
                $next = $imageList[$revList[$image_id] + 1];
            }
            if (isset($imageList[$revList[$image_id] - 1])) {
                $prev = $imageList[$revList[$image_id] - 1];
            }
        }

        $dimensions = $image->getDimensions();

        require ANSEL_TEMPLATES . '/view/headers-image.inc';
        if (!$print_view) {
            require ANSEL_TEMPLATES . '/view/navbar-image.inc';
        }
        require ANSEL_TEMPLATES . '/view/description-image.inc';
        if ($prefs->getValue('showexif')) {
            require_once ANSEL_BASE . '/lib/Exif.php';
            require ANSEL_TEMPLATES . '/view/exif-image.inc';
        }

        if (!$print_view &&
            ($conf['comments']['allow'] == 'all' || ($conf['comments']['allow'] == 'authenticated' && Auth::getAuth())) &&
            $registry->hasMethod('forums/renderThreads') &&
            $registry->hasMethod('forums/postMessage')) {
            $params = array('title' => _("Add a comment"));
            if ($parent = Util::getFormData('parent')) {
                $params['message_parent_id'] = $parent;
            }
            $comments = $registry->call('forums/postMessage', array('ansel', $image_id, 'commentCallback', $params));
            $threads = $registry->call('forums/renderThreads', array($image_id, true, 'ansel', str_replace('%25', '%', Util::addParameter(Horde::selfUrl(true), 'parent', '%p'))));
            if (is_a($threads, 'PEAR_Error')) {
                $threads = $threads->getMessage();
            }
            if (is_a($comments, 'PEAR_Error')) {
                $comments = $comments->getMessage();
            }
            if (!empty($threads)) {
                echo '<br />' . $threads;
            }
            if (!empty($comments)) {
                echo '<br />' . $comments;
            }
        }
    }
    break;

default:
    require_once 'Horde/UI/Pager.php';
    require_once 'Horde/Variables.php';

    $numimages = $gallery->get('num-images');
    $perpage = min($prefs->getValue('imagesperpage'), $conf['thumbnail']['perpage']);
    $pagestart = ($page * $perpage) + 1;
    $pageend = min($numimages, $pagestart + $perpage - 1);
    $imagelist = $gallery->listImages($page * $perpage, $perpage);
    $viewurl = Util::addParameter('view.php', 'gallery', $galleryId);
    $galleries_perpage = $prefs->getValue('galleriesperpage');
    $vars = &Variables::getDefaultVariables();
    $pager = &new Horde_UI_Pager('page', $vars, array('num' => $numimages,
                                                      'url' => $viewurl,
                                                      'perpage' => $perpage));

    $title = $gallery->get('name');
    if (!$print_view) {
        Horde::addScriptFile('popup.js', 'horde', true);
    }
    require ANSEL_TEMPLATES . '/common-header.inc';

    if ($print_view) {
        require $registry->get('templates', 'horde') . '/javascript/print.js';
    } else {
        $print_link = Horde::url(Util::addParameter('view.php', array('gallery' => $galleryId, 'print' => 1)));
        require ANSEL_TEMPLATES . '/menu.inc';

        /* See if we have any parents to go up to. */
        $parent_list = $gallery->getParents();
        $parents = array();
        foreach($parent_list as $id) {
            $p = $ansel_shares->getGallery($id);
            $parents[] = Horde::link(Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $p->getId())), $p->get('name'), 'smallheader') . $p->get('name') . '</a>';
        }
        if (count($parents)) {
            require ANSEL_TEMPLATES . '/view/parents.inc';
        }

        /* Get any sub-galleries. */
        $num_galleries =  Ansel::countGalleries(PERMS_SHOW, false, $gallery->getName(), false);
        $galleries = Ansel::listGalleries(PERMS_SHOW, false, $gallery->getName(), false, $gpage * $galleries_perpage, $galleries_perpage);

        $min = $max = $start = $end = $total = 0;

        if ($num_galleries) {
            $min = $gpage * $galleries_perpage;
            $max = $min + $galleries_perpage;
            if ($max > $num_galleries) {
                $max = $num_galleries - $min;
            }

            $start = $min + 1;
            $end = min($num_galleries, $min + $galleries_perpage);
            $total = 0;

            /* Preserve the parent gallery. */
            $parent = $gallery;
            $parentId = $galleryId;

            $gallery_pager = &new Horde_UI_Pager('gpage', $vars, array('num' => $num_galleries,
                                                                       'url' => $viewurl,
                                                                       'perpage' => $galleries_perpage));

            require ANSEL_TEMPLATES . '/view/subgallery_header.inc';
            require ANSEL_TEMPLATES . '/list/gallery_headers.inc';

            foreach ($galleries as $groupId => $group) {
                foreach ($group['galleries'] as $galleryId => $gallery) {
                    if ($total++ < $min || $total > $max) {
                        continue;
                    }

                    $viewurl = Horde::applicationUrl(Util::addParameter('view.php', 'gallery', $gallery->getId()));
                    $slideurl = Horde::applicationUrl(Util::addParameter('slideshow.php', 'gallery', $gallery->getId()));
                    $galleryurl = Util::addParameter('gallery.php', 'gallery', $gallery->getId());
                    $zipurl = Horde::applicationUrl(Util::addParameter($galleryurl, 'actionID', 'downloadzip'));

                    require ANSEL_TEMPLATES . '/list/gallery_summaries.inc';
                }
            }

            require ANSEL_TEMPLATES . '/list/gallery_footers.inc';
            require ANSEL_TEMPLATES . '/view/subgallery_footer.inc';

            // Restore the parent gallery.
            $gallery = $parent;
            $galleryId = $parentId;
        }
    }

    require ANSEL_TEMPLATES . '/view/gallery.inc';
    break;
}

require $registry->get('templates', 'horde') . '/common-footer.inc';
